SoLTracker = LibStub("AceAddon-3.0"):NewAddon("SoLTracker", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0");
SOLT_VERSION	=	'1.2.2'
local options = {
	type='group',
	args = {
		lock = {
			type = 'toggle',
			name = 'Lock frame',
			desc = 'Toggles the locked state of the SoLTracker frame.',
			get = function()
							return SoLTracker_InfoFrame.lock;
						end,
			set = function()
							SoLTracker_InfoFrame.lock = not SoLTracker_InfoFrame.lock;

							if (SoLTracker_InfoFrame.lock) then
								SoLTracker_InfoFrame:EnableMouse(false);
								SoLTracker_InfoFrame:SetBackdropColor(0,0,0,0);
								
								SoLTracker_InfoFrame:Hide();
								SoLTracker_InfoFrameDuration:SetText('');	
								SoLTracker:Print('Locked');
							else
								SoLTracker_InfoFrame:EnableMouse(true);
								SoLTracker_InfoFrame:SetBackdropColor(0,0,0,.5);
								SoLTracker_InfoFrame:Show();
								SoLTracker_InfoFrameDuration:SetText('SoLTracker');
								SoLTracker:Print('Unlocked, you can move the frame');
							end
						end,
		},
		hideOnCD	=	{
			type = 'toggle',
			name = 'Hide when trinket on CD?',
			desc = 'Toggles if frame is shown when trinket is on cooldown',
			get = function()
							return SoLTracker.db.profile.hideOnCD;
						end,
			set = function()
							SoLTracker.db.profile.hideOnCD = not SoLTracker.db.profile.hideOnCD;

							if (SoLTracker.db.profile.hideOnCD) then
								SoLTracker:Print('Hide frame when trinket on CD');
							else
								SoLTracker:Print('Show frame when trinket on CD');
							end
						end,
		},			
		test = {
			type = 'execute',
			name = 'Test',
			desc = 'Just a little test button :)',
			func = function()
				SoLTracker:CDFeedback()
				SoLTracker:hasTrinketEquipped();
				SoLTracker:Print("Trinket equipped: " .. tostring(SoLTracker.isTrinketEquipped));
				SoLTracker:Print("Can store this amount of heal: " .. SoLTracker.maxStoredHeal);
				SoLTracker:Print("Hide on CD: " .. tostring(SoLTracker.db.profile.hideOnCD));
			end,
		},		
		version = {
			type = 'execute',
			name = 'Version',
			desc = 'Display version information',
			func = function()
						SoLTracker:Print('Current version: ' .. SOLT_VERSION);
					end,
		},		
	}
}


-- init some vars
SoLTracker.isTrinketEquipped	=	false;
SoLTracker.maxStoredHeal	=	0;
SoLTracker.overHeal		=	0;
SoLTracker.expireTime  	=	0;
SoLTracker.trinketItemId	=	false;

function SoLTracker:OnInitialize()
	SoLTracker.db = LibStub("AceDB-3.0"):New("SoLTrackerDB");
	SoLTracker.db:RegisterDefaults({
		profile = {
			hideOnCd = false,
		}
	});

	options.args.profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(SoLTracker.db);
	SoLTracker.configOptions = options;
	LibStub("AceConfig-3.0"):RegisterOptionsTable("SoLTracker", options, {"solt", "sol"});
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("SoLTracker", "|cffFFFFFFScales of Life|r Tracker");
end

function SoLTracker:OnEnable()
	SoLTracker:Print('Enabled');
end

function SoLTracker:setTimerColor(remainingTime)
	if(remainingTime <= 20 and remainingTime >= 10) then
		r = 0;
		g = 1;
		b = 0;
	elseif(remainingTime < 10 and remainingTime >= 5) then
		r = 1;
		g = 0.5;
		b = 0;
	else
		r = 1;
		g = 0;
		b = 0;
	end
	SoLTracker_InfoFrameDuration:SetTextColor(r,g,b,1);		
end;

function SoLTracker:setHealColor(overheal)
	if(overheal >= 12000) then
		r = 0;
		g = 1;
		b = 0;
	elseif(overheal < 12000 and overheal >= 8000) then
		r = 1;
		g = 0.5;
		b = 0;
	else
		r = 1;
		g = 0;
		b = 0;
	end
	SoLTracker_InfoFrameStoredHeal:SetTextColor(r,g,b,1);		
end;


function SoLTracker:TimerFeedback()
	if(not UnitAffectingCombat("player")) then
		self:CancelTimer(self.buffTimer, true);
		if (SoLTracker_InfoFrame:IsVisible()) then	
			SoLTracker_InfoFrame:Hide();
		end;		
		return;
	end;
		
	SoLTracker:getSoLBuff();
	remainingTime	=	SoLTracker:calculateDuration();
	SoLTracker:setHealColor(SoLTracker.overHeal);
	SoLTracker:setTimerColor(remainingTime);

	SoLTracker:getTrinketCD();
	
	if remainingTime == 0 then
		SoLTracker.overHeal = 0;
		SoLTracker:setHealColor(SoLTracker.overHeal);
		--self:CancelTimer(self.buffTimer, true)
	end
	
	if (not SoLTracker_InfoFrame:IsVisible() and not SoLTracker.db.profile.hideOnCD) then	
		SoLTracker_InfoFrame:Show();
	end;
	
	SoLTracker_InfoFrameDuration:SetText(format("%.1f", remainingTime));
	SoLTracker_InfoFrameStoredHeal:SetText(format("%.1f / %.1f", SoLTracker.overHeal / 1000, SoLTracker.maxStoredHeal / 1000) .. "k"	);	
end


function SoLTracker:ShowFrame()
	--SoLTracker:Print('Frame shwon');
	if (not SoLTracker_InfoFrame:IsVisible()) then	
		SoLTracker_InfoFrame:Show();
	end;
end;

function SoLTracker:CDFeedback()
	if(SoLTracker.trinketItemId) then
		if(SoLTracker.db.profile.hideOnCD) then
			self.reshowFrameTimer = self:ScheduleTimer("ShowFrame", 60)
			SoLTracker_InfoFrame:Hide();
			--SoLTracker:Print('Frame hidden');
		end;
	end;
end;

function SoLTracker:OnDisable()
	SoLTracker:CancelAllTimers();
	SoLTracker:Print('Disabled');
end

function SoLTracker:combatLogUnfiltered(eventName, timestamp, eventType, hideCaster, sourceGUID, sourceName, sourceFlags, sourceRaidFlags, destGUID, destName, destFlags, destRaidFlags, spellId, spellName, isOverHeal, healAmount, overHealAmount, param14, param15, param16, ...)
	if (eventType == "SPELL_HEAL" or eventType == "SPELL_PERIODIC_HEAL") and destName == UnitName("player") and self:hasTrinketEquipped() then
		if (spellId == 96880) then -- heal released
			SoLTracker.overHeal 	=	0;
			SoLTracker.expireTime	=	0;
			SoLTracker:CDFeedback();
		end;
		
		if(overHealAmount > 0) then
			SoLTracker.overHeal	=	SoLTracker.overHeal + overHealAmount;
			if(SoLTracker.overHeal > SoLTracker.maxStoredHeal) then
				SoLTracker.overHeal	=	SoLTracker.maxStoredHeal;
			end;
		end;
	end;
end

function SoLTracker:calculateDuration()
	if(SoLTracker.expireTime  == 0) then
		return 0;
	end;
	currentTime 	=	GetTime();
	endTime 		=	SoLTracker.expireTime - currentTime;
	
	digits 		=	1		
	shift 			=	10 ^ digits
	endTime 		=	floor( endTime*shift + 0.5 ) / shift			
	if(endTime == null or endTime == nil or endTime == false or endTime <= 0) then endTime = 0 end;

	return endTime;
end;

function SoLTracker:getTrinketCD()
	startTime, duration, enable = GetItemCooldown(SoLTracker.trinketItemId)

	if(startTime == 0) then
		return 0;
	end;
	
	currentTime 	=	GetTime();
	endTime		=	startTime + duration;
	timeLeft 		=	endTime - currentTime;
	digits 		=	1		
	shift 			=	10 ^ digits
	timeLeft 		=	floor( timeLeft*shift + 0.5 ) / shift	
	
	return timeLeft;
	
end;

function SoLTracker:enterCombat()
    if(self:hasTrinketEquipped()) then
	SoLTracker:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "combatLogUnfiltered");
	self.buffTimer = self:ScheduleRepeatingTimer("TimerFeedback", 0.1)
	CDDuration	= SoLTracker:getTrinketCD();

	if(CDDuration > 0 and SoLTracker.db.profile.hideOnCD) then
		if (SoLTracker_InfoFrame:IsVisible()) then	
			SoLTracker_InfoFrame:Hide();
		end;	
		self.showFrameTimer = self:ScheduleTimer("ShowFrame", CDDuration)
	else
		SoLTracker:ShowFrame();
	end
    else
	SoLTracker:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
	return;
    end;
end

function SoLTracker:getSoLBuff()
	local buffs, i = { }, 1;
	--
	local name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId = UnitBuff("player", i);
	while name do
		if(spellId == 96881 or spellId == 97117) then -- buff for sol normal and hero version
			SoLTracker.expireTime		=	expirationTime;
			break;
		end;
		buffs[#buffs + 1] = buff;
		i = i + 1;
		name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId = UnitBuff("player", i);
	end;
end

function SoLTracker:leaveCombat(eventName, ...)
	SoLTracker:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
	SoLTracker_InfoFrame:Hide();
	SoLTracker:CancelAllTimers();
end

function SoLTracker:hasTrinketEquipped()
	if ( IsEquippedItem(68915) ) then
		SoLTracker.isTrinketEquipped	= true;
		SoLTracker.maxStoredHeal	= 17095;
		SoLTracker.trinketItemId	=  68915;
		
	elseif ( IsEquippedItem(69109)) then
		SoLTracker.isTrinketEquipped	= true;
		SoLTracker.maxStoredHeal	= 19283;
		SoLTracker.trinketItemId	=  69109;
	else
		SoLTracker.isTrinketEquipped	= false;
		SoLTracker.maxStoredHeal	= 0;		
	end;	
	
	return SoLTracker.isTrinketEquipped;
end;


SoLTracker:RegisterEvent("PLAYER_REGEN_ENABLED", "leaveCombat");
SoLTracker:RegisterEvent("PLAYER_REGEN_DISABLED", "enterCombat");
SoLTracker_InfoFrame.lock	= true;